   import java.util.*;
   import java.io.*;
   import java.awt.*;
   import java.awt.event.*;

    public class HangmanGame extends Frame {
      private static final int MAX_EASY_AGE = 8;
      private static final int MAX_MEDIUM_AGE = 11;
    
      private WordList words;
      private Player player;
      private PlayersInfo players;
      private ButtonPanel  buttonPanel;
      private Button [] buttonArray;
      private InputStreamReader inputStream = new InputStreamReader( System.in );
      private BufferedReader stdin = new BufferedReader( inputStream );
      private MessagePanel messagePanel;
      private HangmanWord word;
      private GuessPanel guessPanel;
    
       public HangmanGame() throws IOException {
         inputStream = new InputStreamReader( System.in );
         stdin = new BufferedReader( inputStream );
         players = new PlayersInfo("players.dat");
         setSize( 500, 600 );
         setTitle( "Hangman Game " );
      
         getPlayerInformation();
         getWordList(player.getAge());
         word = new HangmanWord( words.getRandomWord(), player );
         messagePanel = new MessagePanel( );
         add( "Center", messagePanel );
         guessPanel = new GuessPanel();
         add( "South", guessPanel );
      
      } // end HangmanGame constructor
    
     
       protected void getPlayerInformation () throws IOException {
         TextDialog nameDialog = new TextDialog(this, 
            	       "Hi! What's your first name? ");
         nameDialog.show();
         String firstName = nameDialog.response();
      
         nameDialog = new TextDialog(this, 
                "Hi! What's your last name? ");
         nameDialog.show();
         String lastName = nameDialog.response();
      
         if (players.contains(firstName, lastName)) {
            player = players.getPlayer(firstName, lastName);
         } 
         else {
            nameDialog = new TextDialog(this, 
               "When's your birday, mm/dd/yyyy? ");
            nameDialog.show();
            String birthdayString = nameDialog.response();
         
            String [] birthdayFields = birthdayString.split("/");
            Date birthday = new Date();
         // calculate age in milliseconds from jan. 1, 1970
            long age = (long) ((Integer.parseInt(birthdayFields[2]) - 1970)*365.25*24*3600*1000);
            age += ((long) Integer.parseInt(birthdayFields[0]) - 1) * 30 * 24 * 3600*1000;
            age += ((long) Integer.parseInt(birthdayFields[1]) - 1) * 24 * 3600*1000;
            birthday.setTime(age);
            player = new Player(firstName, lastName, birthday);
            players.addPlayer( player );
         } // end if
      
      } // end getPlayerInformation
    
       protected void getWordList(long age) throws IOException {
         if (age <= MAX_EASY_AGE) {
            words = new WordList("easy.txt");
         } 
         else if (age <= MAX_MEDIUM_AGE) {
            words = new WordList("medium.txt");
         } 
         else { // age > MAX_MEDIUM_AGE)
            words = new WordList("hard.txt");
         } // end if
      } // end getWordList
    
       private boolean playAgain()  {
         String [] choices = {"YES", "NO"};
         String [] messages = {"\"" + word.getWordGuessed() + "\" is correct " + player.getFirstName() + "!", 
               "You needed " + word.getNumberOfLettersGuessed() + " letters "
               + "and " + word.getNumberOfIncorrectWordsGuessed() + " word guesses."};
         HumanDialog playsFirstDialog = new HumanDialog( this, 
            			"Would you like to play another game?",
            			choices, messages ); 
         playsFirstDialog.show();
         return playsFirstDialog.response().equals("YES");
      } // end playAgain
    
    
    /*******************************  Inner-classes  ************************/
   
       public class MessagePanel extends Panel
      {
         private String wordsGuessedMessage;
         private String lettersGuessedMessage;
      
          public MessagePanel( )	{
            wordsGuessedMessage = new String( "Number of incorrect word guesses: 0    " );
            lettersGuessedMessage = new String( "Number of letters guessed: 0    ");  
         }
      
          public void paint( Graphics g )
         {
            int height = this.getHeight();
            g.drawString("Number of words guessed: " + word.getNumberOfIncorrectWordsGuessed(),
               10, height/4 );
            g.drawString("Number of letters guessed: " + word.getNumberOfLettersGuessed(), 10, height/2 );
         
            Font currentFont = g.getFont();
            g.setFont( currentFont.deriveFont((float) 20.0) );
            g.drawString(word.getPartialWordGuessed(), 10, (2*height)/3 );  
         
         } // end paint
      } // end inner-class MessagePanel
    
    
       private class GuessPanel extends Panel {
      
          public GuessPanel () {
            setLayout( new GridLayout( 2, 1 ));
         
            ButtonPanel buttonPanel = new ButtonPanel( word.getLettersUnguessed() );
            add( buttonPanel );
         
            WordGuessPanel wordGuessPanel = new WordGuessPanel("Guess a letter or enter a word guess: ");
            add( wordGuessPanel );
         
         } // end GuessPanel constructor
      } // end inner-class GuessPanel
   
       private class ButtonPanel extends Panel {
      
      
          public ButtonPanel(String unguessedLetters ) {
            int numberOfUnguessedLetters = unguessedLetters.length();
            buttonArray = new Button[numberOfUnguessedLetters];
         
            setLayout( new GridLayout( 1, numberOfUnguessedLetters ) );
         
            for (int i = 0; i < numberOfUnguessedLetters; i++ ) {
               buttonArray[i] = new Button( unguessedLetters.substring(i, i+1) );
               buttonArray[i].addActionListener( new LetterButtonListener() );
               add( buttonArray[i] );
            } // end for
         
         } // end ButtonPanel constructor
      
          private class LetterButtonListener implements ActionListener {
             public void actionPerformed( ActionEvent e ) {
               String buttonLetter = e.getActionCommand();
            
               buttonArray[(int) buttonLetter.charAt(0) - (int) 'A'].setLabel("");
               handleGuess( buttonLetter );
            
            } // end actionPerformed
         
         } // end inner-class LetterButtonListener
      
      } // end class ButtonPanel
    
    
       public class WordGuessPanel extends Panel {
         private TextField responseField;
      
          public WordGuessPanel ( String question ) {
            setLayout( new GridLayout( 1, 2) );
         
            add( new Label( question ));
            responseField = new TextField("", 30);
            responseField.addActionListener( new uponEnterKeyPressedListener() );
         
            add(responseField);
         } // end TextDialog constructor
      
          private class uponEnterKeyPressedListener implements ActionListener {
             public void actionPerformed ( ActionEvent e ) {
               String response = responseField.getText();
               responseField.setText("");
               handleGuess( response );
            } // end actionPerformed
         } // end inner class uponEnterKeyPressedListener
      
      } // end class TextDialog
    
       private void handleGuess (String userGuess) {
         word.guess( userGuess );
      
         if (word.isSolved()) {
            messagePanel.repaint();
         
            if (playAgain()) {
            // reset the labels on the letter buttons
               for (char letter = 'A'; letter <= 'Z'; letter++) {
                  buttonArray[letter - 'A'].setLabel("" + letter);
               
               } // end for
               word = new HangmanWord( words.getRandomWord(), player );
            
            } 
            else {
               displayLifeTimeStats();
               System.exit(0);
            } // end if
         
         } // end if
         messagePanel.repaint();
      
      } // end handleGuess
    
       private void displayLifeTimeStats()  {
         String [] choices = {"CLICK WHEN DONE"};
         String [] messages = {player.getFirstName() + " your lifetime statistics are: ",
               "Total Words Solved: " + player.getTotalWordsSolved(),
               "Average Letters Guessed: " + player.getAverageLettersGuessed(),
               "Average Words Guessed: " + player.getAverageWordsGuessed()};
         players.save("players.dat");
      
         HumanDialog playerStatsDialog = new HumanDialog( this, 
            			 "Thanks for playing Hangman!!!",
            			 choices, messages ); 
         playerStatsDialog.show();
      } // end displayLifeTimeStats
    
    
   } // end class HangmanGame